/*
 * c2make.c
 *
 *  Generated on: Jan 20th, 2010
 *  Modified on: Jun 16th, 2010
 *  Author: young
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <time.h>
#include <string.h>

#define MAXBUFSIZE 512

typedef struct {
	char filestr[MAXBUFSIZE];
}DirList;

const char *months[] = {
		"January",
		"February",
		"March",
		"April",
		"May",
		"June",
		"July",
		"August",
		"September",
		"October",
		"November",
		"December"
};

char *get_date_time();
int strend(const char *str, const char *sub);
int check_file_type(const char *filename);
DirList *get_file_names(int *count);
int ctomake(const char *exe, const char *includes,
		const char *libraries, int type);

int main(int argc, char *argv[]) {
	int type;

	if (argc != 4){
		printf("Usage: %s exe includes libraries\n", argv[0]);
		return 1;
	}

	printf("c2make (2.10.06-8)\n");
	printf("C2make Builder, is a helpful assistant tool which can create the Makefile"
			" from the source files directly when you do C, C++, GTK+, Gtkmm, Motif,"
			" wxWidget and OpenGL programming.\n");
	printf("Copyright (C) 2010 Chinsoft Studio.\n\n");
	printf("Written by young <yyhoo2.young@gmail.com> on June 16th, 2010.\n\n");
	printf("--------Menu-------\n");
	printf("1---c program\n");
	printf("2---c++ program\n");
	printf("3---gtk+ program\n");
	printf("4---gtkmm program\n");
	printf("5---motif program\n");
	printf("6---wxWidget program\n");
	printf("7---OpenGL program\n");
	printf("-------------------\n\n");
	printf("please choose a type of the program:\n");
	scanf("%d", &type);
	
	if (ctomake(argv[1], argv[2], argv[3], type) == 0) {
		printf("\ncreate the file makefile successfully!\n");
	} else {
		printf("\ncreate the file makefile failed!\n");
	}

	return 0;
}

/* get the current date time string */
char *get_date_time() {
struct tm lt;
time_t now;
static char timebuf[MAXBUFSIZE];

time(&now);
lt = *localtime(&now);

sprintf(timebuf, "%s %s %dth, %04d", " on", months[lt.tm_mon], lt.tm_mday, lt.tm_year+1900);

return timebuf;
}

/* 
 * the function strend() finds whether the occurrence of 
 * the string sub is in the end of the string str.
 */
int strend(const char *str, const char *sub) {
	int len1 = strlen(str);
	int len2 = strlen(sub);
	
	if (len1 < len2) {
		return 1;
	}
	
	if (strcmp(str + len1 -len2, sub) != 0) {
		return 1;
	}
	
	return 0;
}

int check_file_type(const char *filename) {
	if ((strend(filename, ".c") == 0)||(strend(filename, ".cc") == 0)||
			(strend(filename, ".cxx") == 0)||(strend(filename, ".cpp") == 0)) {
		return 0;
	}
	
	return 1;
}

/*
 *  get the C source file names of the current directory 
 */
DirList *get_file_names(int *count) {
	DirList *dirlist = NULL;
	int n = 0;
	DIR *pdir = NULL;
	struct dirent *pent = NULL;

	if ((pdir = opendir(".")) == NULL) {
		fprintf(stderr, "open dir failed.\n");
		*count = 0;
		return NULL;
	}

	dirlist = (DirList *)malloc(1 * sizeof(DirList));
	
	while ((pent = readdir(pdir)) != NULL) {
		if (check_file_type(pent->d_name) == 0) {
			n++;
			dirlist = (DirList *)realloc(dirlist, (n+1) * sizeof(DirList));
			strcpy(dirlist[n-1].filestr, pent->d_name);
		}
	}
	
	*count = n;
	
	closedir(pdir);
	
	return dirlist;
}

/* 
 * create the Makefile from the source files directly
 */
int ctomake(const char *exe, const char *includes,
		const char *libraries, int type) {
	FILE *fp = NULL;
	char *protype = NULL;  /* the program type */
	char *compiler = NULL; /* the c compiler */
	char *lib = NULL;      /* the library files */
	char *token = NULL;
	char *inc = NULL;      /* the include files */
	char *p = NULL;
	DirList *plist = NULL;
	int i = 0, count = 0;

	switch (type) {
	case 1:
		protype = "c";
		compiler = "gcc";
		inc = "";
		lib = "";
		break;
	case 2:
		protype = "c++";
		compiler = "g++";
		inc = "";
		lib = "";
		break;
	case 3:
		protype = "gtk+";
		compiler = "gcc";
		inc = "`pkg-config gtk+-2.0 --cflags` ";
		lib = "`pkg-config gtk+-2.0 --libs` ";
		break;
	case 4:
		protype = "gtkmm";
		compiler = "gcc";
		inc = "`pkg-config gtkmm-2.4 --cflags` ";	
		lib = "`pkg-config gtkmm-2.4 --libs` ";
		break;
	case 5:
		protype = "motif";
		compiler = "gcc";
		inc = "";
		lib = "-lXm -lXt -lX11 ";
		break;
	case 6:
		protype = "wxWidget";
		compiler = "g++";
		inc = "`wx-config --cxxflags` ";
		lib = "`wx-config --libs` ";
		break;
	case 7:
		protype = "openGL";
		compiler = "gcc";
		inc = "";
		lib = "-lGL -lGLU -lglut ";
		break;
	default:
		printf("error program type.\n");
		return 1;
	}
	
	plist = get_file_names(&count); /* get the name strings of the c source files */
	if (count == 0) {
		if (plist != NULL) {
			free(plist);
			plist = NULL;
		}
		return 1;
	}else if (count == 1) {
		token = "";
	} else {
		token = " \\";
	}
	
	if ((fp = fopen("makefile", "w")) == NULL) {
		printf("create the file makefile failed.\n");
		return 1;
	}

	fprintf(fp, "%s%s%s%s\n", "# this is a makefile of the ", protype, " project ", exe);
	fprintf(fp, "%s%s\n", "# the standard c compiler on GNU linux is ", compiler);
	fprintf(fp, "%s%s%s\n\n", "# written by ", (char *)getenv("USER"), get_date_time());

	fprintf(fp, "%30s\n\n", "TARGET = .");
	fprintf(fp, "%30s%s\n", "GCC = ", compiler);
	fprintf(fp, "%30s%s\n", "CFLAGS = ", "-g -Wall");
	fprintf(fp, "%30s%s\n\n", "CFLAGC = ", "-c");
	
	for (i = 0; i < count; i++) {

		if (i == 0) {
			fprintf(fp, "%30s%s%s\n", "SRC = ", plist[i].filestr, token);
		} else if (i == count -1) {
			 fprintf(fp, "%30s%s\n", " ", plist[i].filestr);
		} else {
			 fprintf(fp, "%30s%s%s\n", " ", plist[i].filestr, token);
		}
	}
	
	for (i = 0; i < count; i++) {
		p = strstr(plist[i].filestr, ".");
		*(++p) = 'o'; 
		*(++p) = '\0';

		if (i == 0) {
			fprintf(fp, "%30s%s%s\n", "OBJS = ", plist[i].filestr, token);
		} else if (i == count -1) {
			 fprintf(fp, "%30s%s\n", " ", plist[i].filestr);
		} else {
			 fprintf(fp, "%30s%s%s\n", " ", plist[i].filestr, token);
		}
	}
	
	fprintf(fp, "%30s%s\n\n", "EXEC = ", exe);

	fprintf(fp, "%30s%s%s\n", "INCLUDE = ", inc, includes);
	fprintf(fp, "%30s%s%s\n\n", "LIBS = ", lib, libraries);

	fprintf(fp, "%s\n\n", "all: $(EXEC)");
	fprintf(fp, "%s\n", "$(EXEC): $(OBJS)");
	fprintf(fp, "%s\n", "\t$(GCC) $(CFLAGS) $(OBJS) $(LIBS) -o $@");
	fprintf(fp, "%s\n\n", "\trm -f $(OBJS)");
	
	fprintf(fp, "%s\n", "$(OBJS): $(SRC)");
	fprintf(fp, "%s\n\n", "\t$(GCC) $(CFLAGC) $(SRC) $(INCLUDE)");

	fprintf(fp, "%s\n", "install:");
	fprintf(fp, "%s\n\n", "\tcp $(EXEC) /usr/local/bin");
	
	fprintf(fp, "%s\n", "uninstall:");
	fprintf(fp, "%s\n\n", "\trm -f /usr/local/bin/$(EXEC)");
	
	fprintf(fp, "%s\n", "clean:");
	fprintf(fp, "%s\n", "\trm -f $(EXEC)");

	fclose(fp);
	
	if (plist != NULL) { /* be careful, you must free the pointer after the function finished */
		free(plist);
		plist = NULL;
	}

	return 0;
}
